function install_rapidstm32

% Check Matlab and other tools release/version.
%
% Install RapidSTM32 toolsuite if the minimum requirements are met.
%
% Minimum Requirements
%
% - Matlab Release 2009a
% - Simulink
% - Real-Time Workshop or Simulink Coder
% - Real-Time Workshop Embedded Coder or Embedded Coder
%
% RapidSTM32 will be installed in the root directory of the same drive as
% Matlab root.

%% Installation confirmation
choice = questdlg('Do you want to proceed with RapidSTM32 Blockset Installation?',...
    'RapidSTM32 Blockset Installation','Yes','No','Yes');
if strcmp(choice,'No')
    error('RapidSTM32 Blockset installation was canceled.')
end

disp('Proceeding...')
pause(0.5)

%% Check existing software versions
disp('Checking existing software version...')
ver_txt = struct2cell(ver);

% list of require tools, version, release
tool_list{1} = 'MATLAB';
tool_list{2} = 'Simulink';
tool_list{3} = 'Real-Time Workshop';
tool_list{4} = 'Real-Time Workshop Embedded Coder';
release_list{1} = 2009;
release_list{2} = 2009;
release_list{3} = 2009;
release_list{4} = 2009;
version_list{1} = 7.8;
version_list{2} = 7.3;
version_list{3} = 7.3;
version_list{4} = 5.3;
check_release_list = 1; % index to product to check for release otherwise check version
str = cell(7,1);

idx = find(strcmp('MATLAB',ver_txt(1,:))==1);
installed_release = regexp(ver_txt(3,idx),'(R\w*)','match');
installed_release = cell2mat(installed_release{1});
installed_release = str2double(installed_release(2:end-1));
if (installed_release <= 2010)
    for k = 1:size(tool_list,2)
        tool_name = tool_list{k};
        required_release = release_list{k};
        required_version = version_list{k};
        idx = find(strcmp(tool_name,ver_txt(1,:))==1);
        if ~isempty(idx)
            installed_release = regexp(ver_txt(3,idx),'(R\w*)','match');
            installed_release_full = cell2mat(installed_release{1});
            installed_version_full = str2num(cell2mat(ver_txt(2,idx)));
            switch tool_name
                case 'MATLAB'
                    installed_matlab_ver = installed_version_full;
            end
            % check release
            if ~isempty(find(k == check_release_list, 1))
                if (str2num(installed_release_full(2:end-1)) < required_release)
                    str{1} = 'RapidSTM32 requires:';
                    str{2} = '';
                    str{3} = [tool_name ' Release: R' num2str(required_release) ' or later.'];
                    str{4} = '';
                    str{5} = ['Your ' tool_name ' installation is Release: ' installed_release_full '.'];
                    str{6} = '';
                    str{7} = 'Installation is not complete and will now terminate.';
                    errordlg(str,'Installation Error','modal')
                    return;
                end
            else
                % check version
                if (installed_version_full < required_version)
                    str{1} = 'RapidSTM32 requires:';
                    str{2} = '';
                    str{3} = [tool_name ' Version: ' num2str(required_version) ' or later.'];
                    str{4} = '';
                    str{5} = ['Your ' tool_name ' installation is Version: ' num2str(installed_version_full,'%2.1f') '.'];
                    str{6} = '';
                    str{7} = 'Installation is not complete and will now terminate.';
                    errordlg(str,'Installation Error','modal')
                    return;
                end
            end
        else
            if ~isempty(find(k == check_release_list, 1))
                str{1} = 'RapidSTM32 requires:';
                str{2} = '';
                str{3} = [tool_name ' Release: R' num2str(required_release) ' or later.'];
                str{4} = '';
                str{5} = ['Your Matlab installation does not have ' tool_name ' installed.'];
                str{6} = '';
                str{7} = 'Installation is not complete and will now terminate.';
                errordlg(str,'Installation Error','modal')
            else
                str{1} = 'RapidSTM32 requires:';
                str{2} = '';
                str{3} = [tool_name ' Version: ' num2str(required_version) ' or later.'];
                str{4} = '';
                str{5} = ['Your Matlab installation does not have ' tool_name ' installed.'];
                str{6} = '';
                str{7} = 'Installation is not complete and will now terminate.';
                errordlg(str,'Installation Error','modal')
            end
            return;
        end
    end
end
disp('Checking software version OK.')

%% Determine matlabroot drive
drv = matlabroot;
drv = drv(1:2);
rapidstm32root = fullfile(drv,'rapidstm32');

%% Remove previous installation
pidir = fullfile(drv,'rapidstm32'); % previous_installation_directory
if exist(pidir,'dir')
    choice = questdlg(['Previous installation of RapidSTM32 Blockset is detected at ' pidir char(10)...
        'This installation needs to remove that directory completely.' char(10)...
        'Any files in that directory will deleted and can not be recovered.' char(10)...
        'Please make sure to back up any of your work files.' char(10)...
        'Are you sure you want to proceed?'],...
        'Warning!!',...
        ['Proceed and delete ' pidir],'Do not proceed','Do not proceed');
    if strcmp(choice,['Proceed and delete ' pidir])
        %% Uninstall previous installation
        uninstall_rapidstm32(1);
    else
        install_notfinish_str = 'RapidSTM32 Blockset Installation was not successful.';
        disp(install_notfinish_str)
        uiwait(helpdlg(install_notfinish_str,'Warning!!'));
        cd(pidir)
        return
    end
end

%% Unpack installation package
disp('Unpacking installation package...')
fdir = dir('rapidstm32_install.dat');
if isempty(fdir)
    str = ['Unable to find "rapidstm32_install.dat". ' char(10) ...
        'Most likely cause: you have not unzipped the whole installation package. ' ...
        'or you try to run install_rapidstm32.m from "INSIDE" a zip program.' char(10)...
        'Please try unzip the whole package and run install_rapidstm32.m again.'];
    error(str)
else
    eval(['!"' fullfile(pwd,'7za465') '\7za" x rapidstm32_install.dat -o' rapidstm32root '\']);
    disp('Unpacking OK.')
end

%% Install RapidSTM32 specifics
cd(fullfile(rapidstm32root, 'rapidstm32', 'src'))
install_rapidstm32_specifics(drv)

%% .NET suggestion
warningstr = (['You may check whether Microsoft .NET Framework is installed and which versions (if any) ' ...
    'by running the following code ' char([10 10])...
    'javascript:alert(navigator.userAgent) ' char([10 10])...
    'from Internet Explorer URL edit box. '...
    'At least .NET Framework version 3.5 is required.']);
disp(warningstr);

%% Installation complete
install_finish_str = 'Congratulation! RapidSTM32 Blockset Installation Complete. Please do not forget to install to install Keil RealView MDK for ARM and .NET Framework (If you have not done so).';
disp(install_finish_str)
uiwait(helpdlg(install_finish_str,'RapidSTM32 Installation Complete'));
cd(pidir)
